/*
 * Copyright (c) 2009, Sun Microsystems, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of Sun Microsystems, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


/*
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

#include <sys/types.h>
#include <sys/socket.h>

#include <assert.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>

#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>

int
getrpcport(host, prognum, versnum, proto)
	char *host;
	int prognum, versnum, proto;
{
	struct sockaddr_in addr;
	struct addrinfo hints, *result, *rp;
	int ret = 0;

	assert(host != NULL);
	memset(&hints, 0, sizeof(struct addrinfo));
	hints.ai_family = AF_INET; /* ??? :-( */
#if 0
#ifdef AI_V4MAPPED
	hints.ai_flags |= AI_V4MAPPED;
#endif
#ifdef AI_ADDRCONFIG
	hints.ai_flags |= AI_ADDRCONFIG;
#endif
#endif
	if (getaddrinfo(host, NULL, &hints, &result) != 0)
		return (0);
	for (rp = result; rp != NULL; rp = rp->ai_next) {
		assert (rp->ai_family == AF_INET && rp->ai_addrlen == 16);
		memcpy(&addr, rp->ai_addr, rp->ai_addrlen);
		assert (addr.sin_family == AF_INET && addr.sin_port == 0);
		/* Inconsistent interfaces need casts! :-( */
		ret = (pmap_getport(&addr, (u_long)prognum, (u_long)versnum,
		    (u_int)proto));
		if (ret)
			break;
	}
	freeaddrinfo(result);
	return (ret);
}
